// MutAirMail.uc
// Mutator class and base functions for Air Mail.

// This is the actual core of the Air Mail game. It ensures that AirMail packages are regularly added
// to the map and that the total number of Air Mail packages, in inventory and on the ground, is
// always equal to MAX_PICKUPS.

class MutAirMail extends xMutator
	config(AirMail);

var config float SLEEP_TIME; // amount of time to wait between performing inventory on runes
var config int MAX_AIRMAIL; // Number of Air Mail powerups available. Must be defined below in defaultproperties.

var config int MAX_PICKUPS; // Max number of Air Mail pickups/inventory allowed at a time

// Do not confuse MAX_AIRMAIL and MAX_PICKUPS. MAX_AIRMAIL is the number of different Air Mail powerups.
// MAX_PICKUPS is the max number of packages we want in the level at any given time.

var config array<string> Runes; // String array of air mail pickup classes. Make this length the same as MAX_AIRMAIL
var array<vector> GunSpots;

// Ensures our AirMailRules come into effect.
function PostBeginPlay()
{
	local AirMailRules foo;

	SetupGunSpots();
	foo = spawn(class'AirMailRules');
	if (Level.Game.GameRulesModifiers == None)
		Level.Game.GameRulesModifiers = foo;
	else
		Level.Game.GameRulesModifiers.AddGameRules(foo);
	SetTimer(SLEEP_TIME, true);
}

// Set up an array of valid locations to spawn airmails.
// Might want to use Level.NavigationPointList instead...
// If any pathnodes have the tag "airmail", airmails will be spawned ONLY at those pathnodes
// otherwise all pathnodes and PlayerStarts are game
function SetupGunSpots()
{
	local PathNode pn;
	local PlayerStart ps;
	local int NumFlavins, Countdown;

	if (GunSpots.Length > 0)
		GunSpots.Remove(0, GunSpots.Length);
	NumFlavins = 0;
	// Look first for PathNodes with the "AirMail" tag
	foreach AllActors(class'PathNode', pn) {
		if (Instr(pn.tag,"airmail") >= 0)
		{
			GunSpots.Insert(GunSpots.Length, 1.0);
			GunSpots[GunSpots.Length-1] = pn.Location;
			NumFlavins++;
		}
		//log("Added PathNode " @ pn @ " to gun spot list");
	}
	if (NumFlavins == 0) { // No AMN's found, use regular nodes instead.
		// Adds all pathnodes to the list
		foreach AllActors(class'PathNode', pn) {
			GunSpots.Insert(GunSpots.Length, 1.0);
			GunSpots[GunSpots.Length-1] = pn.Location;
			NumFlavins++;
			//log("Added PathNode " @ pn @ " to gun spot list");
		}
		// Adds all playerstarts to the list
		foreach AllActors(class'PlayerStart', ps) {
			GunSpots.Insert(GunSpots.Length, 1.0);
			GunSpots[GunSpots.Length-1] = ps.Location;
			NumFlavins++;
			//log("Added PlayerStart " @ ps @ " to gun spot list");
		}
	}
}

// RandomSpot()
// Returns a random path node or playerstart in the level.
// Copied straight from Golden Gun game type.
function vector RandomSpot()
{
	local int NumFlavins, Countdown;

	NumFlavins = GunSpots.Length;
	Countdown=rand(NumFlavins);

	return GunSpots[Countdown];
}

function vector FindDropPoint(vector SourceLoc)
{
	local Actor HitActor;
	local vector HitLocation, HitNormal, checkpos;
	local Rotator userot;
	local vector NewDropPoint;

	checkpos = SourceLoc;
	checkpos.z += 600;

	HitActor = Trace(HitLocation, HitNormal, checkpos, SourceLoc, false);
	if(HitActor != None)
	{
		HitLocation+=CollisionHeight*HitNormal;
		NewDropPoint=HitLocation;
	}
	else
		NewDropPoint=CheckPos;

	return NewDropPoint;
}

// See if we need to spawn any more air mail pickups.
function DoInventory()
{
	local int RunesFound; // number of pickups/inventory found
	local AirMailInv ami;
	local AirMailPickup amp;
	local int RunesToSpawn; // Number of pickups we need to spawn
	local int RunesSpawned; // Number of pickups spawned
	local vector PlaceLoc;
	local int RandRune;
	local class<AirMailPickup> PickupToSpawn;

	// Check for air mails being held by players
	foreach DynamicActors(class'AirMailInv', ami) {
		RunesFound++;
	}
	// Check for pickups on the ground
	foreach DynamicActors(class'AirMailPickup', amp) {
		RunesFound++;
	}
	//log ("Found" @ RunesFound @ "airmails in level," @ MAX_PICKUPS @ "required");
	if (RunesFound < MAX_PICKUPS) {
		RunesToSpawn = MAX_PICKUPS - RunesFound;
		// Spawn extra pickups
		for(RunesSpawned = 1; RunesSpawned <= RunesToSpawn; RunesSpawned++)
		{
			PlaceLoc = RandomSpot();
			RandRune = Rand(MAX_AIRMAIL);
			PickupToSpawn = class<AirMailPickup>(DynamicLoadObject(Runes[RandRune], class'Class'));
			//log ("Attempting to place" @ PickupToSpawn @ "at" @ PlaceLoc);
			PlaceLoc = FindDropPoint(PlaceLoc);
			amp = spawn(PickupToSpawn,,,PlaceLoc);
			if (amp == None)
				warn("Couldn't drop" @ PickupToSpawn @ "at" @ PlaceLoc);
			else
			{
				amp.FindGround();
				spawn(class'AirMailAppearEffects',,,amp.location);
			}
		}
	}
}

event Timer()
{
	DoInventory();
}

/*
// This is the core state of MutAirMail. It continually checks for air mail packages.
// Checking is done at an interval of SLEEP_TIME seconds. Defaults to 60.
auto state Active
{
Begin:
	DoInventory();
	sleep(SLEEP_TIME);
	goto('Begin');
}*/

defaultproperties
{
	Group="AirMail"
	FriendlyName="Air Mail (1.1 Final)"
	Description="Special delivery by air mail! Randomly drops packages that provide various boosts."
	Runes[0]="AirMail.amAirJordanPickup"
	Runes[1]="AirMail.amArcadeMachinePickup"
	Runes[2]="AirMail.amAutoAmmoPickup"
	Runes[3]="AirMail.amAutoArmorPickup"
	Runes[4]="AirMail.amBeansPickup"
	Runes[5]="AirMail.amBettyPickup"
	Runes[6]="AirMail.amDoghousePickup"
	Runes[7]="AirMail.amFakeAirmailPickup"
	Runes[8]="AirMail.amFakeFoodPickup"
	Runes[9]="AirMail.amFirebombPickup"
	Runes[10]="AirMail.amGrav2Pickup"
	Runes[11]="AirMail.amHyperspacePickup"
	Runes[12]="AirMail.amIcarusPickup"
	Runes[13]="AirMail.amJihadPickup"
	Runes[14]="AirMail.amLawyerPickup"
	Runes[15]="AirMail.amOneRingPickup"
	Runes[16]="AirMail.amPandoraPickup"
	Runes[17]="AirMail.amRegenPickup"
	Runes[18]="AirMail.amRepelPickup"
	Runes[19]="AirMail.amShadowPickup"
	Runes[20]="AirMail.amSpringPickup"
	Runes[21]="AirMail.amStarmanPickup"
	Runes[22]="AirMail.amSubspacePickup"
	Runes[23]="AirMail.amSuperHealthPickup"
	Runes[24]="AirMail.amTaiKwanLeapPickup"
	Runes[25]="AirMail.amTeleporterPickup"
	Runes[26]="AirMail.amToughSkinPickup"
	Runes[27]="AirMail.amVengeancePickup"
	Runes[28]="AirMail.amWeightGainPickup"
	Runes[29]="AirMail.amWeirdPickup"
	Runes[30]="AirMail.amZintharPickup"
	MAX_PICKUPS=16
	SLEEP_TIME=5
	MAX_AIRMAIL=31
}